#ifndef _COURSE_CPP
#define _COURSE_CPP

#include <string>
#include <fstream>
#include "Course.h"

using namespace std;

Course::Course( string aCode )
{
   strncpy( cCode, aCode.c_str(), cCodeSz-1 );
                                          cCode[cCodeSz-1] = '\0';
   cTitle[0] = '\0';
   cFSem = 0;
   cCompuls = false;
   cCateg[0] = '\0';
   cWH = 0;
   cUnits = 0;
   cNoOfStudents = 0;
} // Course::Course

//-----------------------------------------------------------------
//                             
//-----------------------------------------------------------------
void Course::setCode( string aCode )
{
   if ( (aCode.length() > 0) && (aCode == cPrereq) )
      throw CourseXptn( "setCode", CourseXptn::autoRef,
                        aCode.c_str() );
   strncpy( cCode, aCode.c_str(), cCodeSz-1 );
                                          cCode[cCodeSz-1] = '\0';
} // Course::setCode

void Course::setTitle( string aTitle )
{
   strncpy( cTitle, aTitle.c_str(), cTitleSz-1 );
                                        cTitle[cTitleSz-1] = '\0';
} // Course::setTitle

void Course::setFSem( int aFSem )
{
   if ( aFSem < 1 || 8 < aFSem)
      throw CourseXptn( "setFSem",
                        CourseXptn::rangeError, aFSem );
   cFSem = aFSem;
} // Course::setFSem

void Course::setCateg( string aCateg )
{
   strncpy( cCateg, aCateg.c_str(), cCategSz-1 ); 
                                        cTitle[cCategSz-1] = '\0';
} // Course::setCateg

void Course::setPrereq( const string& prCode )
{
   if ( (prCode.length() > 0) && (prCode == cCode) )
      throw CourseXptn( "setPrereq", CourseXptn::autoRef,
                        prCode.c_str() );
   strncpy( cPrereq, prCode.c_str(), cCodeSz-1 );
                                        cPrereq[cCodeSz-1] = '\0';
} // Course::setPrereq

void Course::save( ostream& bout ) const
{
   if ( bout.fail() )
      throw CourseXptn( "save", CourseXptn::fileNotOpen );
   bout.write( cCode, sizeof(cCode) );        //  
   bout.write( cTitle, sizeof(cTitle) );       //  
   bout.write( reinterpret_cast<const char*>(&cFSem),
               sizeof(cFSem) );                  //  
   bout.write( reinterpret_cast<const char*>(&cCompuls),
               sizeof(cCompuls) );       //   
   bout.write( &cSector, sizeof(cSector) );              // 
   bout.write( cCateg, sizeof(cCateg) );              // 
   bout.write( reinterpret_cast<const char*>(&cWH), sizeof(cWH) );
                                              //   
   bout.write( reinterpret_cast<const char*>(&cUnits),
               sizeof(cUnits) );             //  
   bout.write( cPrereq, sizeof(cPrereq) );       //  
   bout.write( reinterpret_cast<const char*>(&cNoOfStudents),
               sizeof(cNoOfStudents) );  // . .  
   if ( bout.fail() )
      throw CourseXptn( "save", CourseXptn::cannotWrite );
} // Course::save

void Course::load( istream& bin )
{
   bin.read( cCode, cCodeSz );                 //  
   if ( !bin.eof() )
   { 
      bin.read( cTitle, cTitleSz );             //  
      bin.read( reinterpret_cast<char*>(&cFSem), sizeof(cFSem) );
                                                 //  
      bin.read( reinterpret_cast<char*>(&cCompuls),
                                               sizeof(cCompuls) );
                                         //   
      bin.read( &cSector, sizeof(cSector) );             // 
      bin.read( cCateg, cCategSz );                    // 
      bin.read( reinterpret_cast<char*>(&cWH), sizeof(cWH) );
                                              //   
      bin.read( reinterpret_cast<char*>(&cUnits), sizeof(cUnits));
                                             //  
      bin.read( cPrereq, cCodeSz );               // 
      bin.read( reinterpret_cast<char*>(&cNoOfStudents),
                sizeof(cNoOfStudents) ); // . .  
      if ( bin.fail() )
         throw CourseXptn( "load", CourseXptn::cannotRead );
   } // if ( !bin.eof() ). . .
} // Course::load

void Course::display( ostream& tout ) const
{
   if ( tout.fail() )
      throw CourseXptn( "display", CourseXptn::fileNotOpen );
   tout << cCode << '\t' << cTitle << '\t';
   if ( cCompuls )  tout << "\t";
              else  tout << "\t";
   tout << cSector << '\t' << cCateg << '\t' << cWH << '\t'
        << cUnits << '\t' << cPrereq << '\t' << cFSem << '\t'
        << cNoOfStudents << endl;
   if ( tout.fail() )
      throw CourseXptn( "display", CourseXptn::cannotWrite );
} // Course::display

#endif // _COURSE_CPP
